@@ -1,4 +1,4 @@
-# Copyright (c) 2003-2004 Chris Ridd <chris.ridd@isode.com> and
+# Copyright (c) 2003 Chris Ridd <chris.ridd@isode.com> and
 # Graham Barr <gbarr@pobox.com>. All rights reserved.  This program is
 # free software; you can redistribute it and/or modify it under the
 # same terms as Perl itself.
@@ -7,12 +7,11 @@ package Net::LDAP::RootDSE;
 
 use Net::LDAP::Entry;
 
-our @ISA = qw(Net::LDAP::Entry);
-our $VERSION = "0.02";
+@ISA = qw(Net::LDAP::Entry);
+$VERSION = "0.01";
 
 use strict;
 
-sub supported_feature        { _supported_feature( @_, 'supportedFeatures'       ) }
 sub supported_extension      { _supported_feature( @_, 'supportedExtension'      ) }
 sub supported_version        { _supported_feature( @_, 'supportedLDAPVersion'    ) }
 sub supported_control        { _supported_feature( @_, 'supportedControl'        ) }
@@ -40,7 +39,7 @@ Net::LDAP::RootDSE - An LDAP RootDSE object
  my $dse = $ldap->root_dse();
 
  # get naming Contexts
- my @contexts = $dse->get_value('namingContexts');
+ my @contexts = $dse->get_value('namingContext');
 
  # get supported LDAP versions as an array reference
  my $versions = $dse->get_value('supportedLDAPVersion', asref => 1);
@@ -60,11 +59,6 @@ C<get_value> is identical to L<Net::LDAP::Entry/get_value>
 Returns true if the server supports all of the specified
 extension OIDs
 
-=item supported_feature ( OID_LIST )
-
-Returns true if the server supports all of the specified
-feature OIDs
-
 =item supported_version ( VERSION_LIST )
 
 Returns true if the server supports all of the specified
@@ -88,13 +82,11 @@ L<Net::LDAP>, L<Net::LDAP::Entry>
 
 =head1 AUTHOR
 
-Chris Ridd E<lt>chris.ridd@isode.comE<gt>,
+Chris Ridd E<lt>chris.ridd@isode.comE<gt>, 
 Graham Barr E<lt>gbarr@pobox.comE<gt>.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003-2004, Chris Ridd and Graham Barr. All rights reserved. This
+Copyright (c) 2003, Chris Ridd and Graham Barr. All rights reserved. This
 library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
-
-=cut
