@@ -1,6 +1,7 @@
-# Copyright (c) 1999-2000 Graham Barr <gbarr@pobox.com>. All rights reserved.
-# This program is free software; you can redistribute it and/or
-# modify it under the same terms as Perl itself.
+# Copyright (c) 1999-2002 Graham Barr <gbarr@pobox.com> and
+# Norbert Klasen <norbert.klasen@daasi.de> All Rights Reserved.
+# This program is free software; you can redistribute it and/or modify
+# it under the same terms as Perl itself.
 
 package Net::LDAP::Util;
 
@@ -32,14 +33,16 @@ the L<Net::LDAP> modules.
 
 use vars qw($VERSION);
 require Exporter;
+require Net::LDAP::Constant;
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(
   ldap_error_name
   ldap_error_text
   ldap_error_desc
   canonical_dn
+  ldap_explode_dn
 );
-$VERSION = "0.06";
+$VERSION = "0.10";
 
 =item ldap_error_name ( NUM )
 
@@ -49,24 +52,7 @@ is returned.
 
 =cut
 
-my @err2name;
-
-sub ldap_error_name {
-  my $code = 0+ shift;
-  require Net::LDAP::Constant;
-
-  unless (@err2name) {
-    local *FH;
-
-    if (open(FH,$INC{'Net/LDAP/Constant.pm'})) {
-      while(<FH>) {
-        ($err2name[hex($2)] = $1) if /^sub\s+(LDAP_\S+)\s+\(\)\s+\{\s+0x([0-9a-fA-f]{2})\s+\}/;
-      }
-      close(FH);
-    }
-  }
-  $err2name[$code] || sprintf("LDAP error code %d(0x%02X)",$code,$code);
-}
+# Defined in Constant.pm
 
 =item ldap_error_text ( NUM )
 
@@ -75,40 +61,7 @@ error code given is unknown then C<undef> is returned.
 
 =cut
 
-sub ldap_error_text {
-  my $name = ldap_error_name(shift);
-  my $text;
-  if($name =~ /^LDAP_/) {
-    my $pod = $INC{'Net/LDAP/Constant.pm'};
-    substr($pod,-3) = ".pod";
-    local *F;
-    open(F,$pod) or return;
-    local $/ = "";
-    local $_;
-    my $len = length($name);
-    my $indent = 0;
-    while(<F>) {
-      if(substr($_,0,11) eq "=item LDAP_") {
-        last if defined $text;
-	$text = "" if /^=item $name\b/;
-      }
-      elsif(defined $text && /^=(\S+)/) {
-        $indent = 1 if $1 eq "over";
-        $indent = 0 if $1 eq "back";
-	$text .= " * " if $1 eq "item";
-      }
-      elsif(defined $text) {
-        if($indent) {
-          s/\n(?=.)/\n   /sog;
-	}
-        $text .= $_;
-      }
-    }
-    close(F);
-    $text =~ s/\n+\Z/\n/ if defined $text;
-  }
-  $text;
-}
+# Defined in Constant.pm
 
 =item ldap_error_desc ( NUM )
 
@@ -224,33 +177,29 @@ sub ldap_error_desc {
 
 
 
-=item canonical_dn ( DN )
 
-Returns the given DN in a canonical form. Returns undef if DN is
-not a valid Distinguished Name
 
-It performs the following operations on the given DN
+=item canonical_dn ( DN [ , OPTIONS ] )
+
+Returns the given B<DN> in a canonical form. Returns undef if B<DN> is
+not a valid Distinguished Name. (Note: The empty string "" is a valid DN.)
+B<DN> can either be a string or reference to an array of hashes as returned by 
+ldap_explode_dn, which is useful when constructing a DN.
+
+It performs the following operations on the given B<DN>:
 
 =over 4
 
 =item *
 
-Lowercases values that are # followed by hex.
-
-=item *
-
-Uppercases type names.
-
-=item *
-
-Removes the leading OID. characters if the type is an OID instead
+Removes the leading 'OID.' characters if the type is an OID instead
 of a name.
 
 =item *
 
-Escapes all RFC 2253 special characters, and any other character
-where the ASCII code is <32 or >= 127, with a backslash and a two
-digit hex code.
+Escapes all RFC 2253 special characters (",", "+", """, "\", "E<lt>",
+"E<gt>", ";", "#", "=", " "), slashes ("/"), and any other character
+where the ASCII code is E<lt> 32 as \hexpair.
 
 =item *
 
@@ -259,82 +208,284 @@ Converts all leading and trailing spaces in values to be \20.
 =item *
 
 If an RDN contains multiple parts, the parts are re-ordered so that
-the attribute names are in alphabetical order.
+the attribute type names are in alphabetical order.
 
 =back
 
-B<Note> values that are hex encoded (ie start with a #) are not
-decoded. So C<SN=Barr> is not treated the same as C<SN=#42617272>
+B<OPTIONS> is a list of name/value pairs, valid options are:
+
+=over 4
+
+=item casefold
+
+Controls case folding of attribute type names. Attribute values are not
+affected by this option. The default is to uppercase. Valid values are:
+	
+=over 4
+
+=item lower
+
+Lowercase attribute type names.
+
+=item upper
+
+Uppercase attribute type names. This is the default.
+
+=item none
+
+Do not change attribute type names.
+
+=back
+
+=item mbcescape
+
+If TRUE, characters that are encoded as a multi-octet UTF-8 sequence 
+will be escaped as \(hexpair){2,*}.
+
+=item reverse
+
+If TRUE, the RDN sequence is reversed.
+
+=item separator
+
+Separator to use between RDNs. Defaults to comma (',').
+
+=back
 
 =cut
 
+sub canonical_dn($%) {
+  my ($dn, %opt) = @_;
 
-sub canonical_dn {
-  my $dn = shift;
-  $dn = $dn->dn if ref($dn);
+  return $dn unless defined $dn and $dn ne '';
   
-  my (@dn, @rdn);
+  # create array of hash representation
+  my $rdns = ref($dn) eq 'ARRAY'
+		? $dn
+		: ldap_explode_dn( $dn )
+    or return undef; #error condition
+  
+  # assign specified or default separator value
+  my $separator = $opt{separator} || ',';
+
+  # flatten all RDNs into strings
+  my @flatrdns =
+    map {
+      my $rdn = $_;
+      my @types = sort keys %$rdn;
+      join('+',
+        map {
+          my $val = $rdn->{$_};
+          
+          if ( ref($val) ) {
+            $val = '#' . unpack("H*", $$val);
+          } else {
+            #escape insecure characters and optionally MBCs
+            if ( $opt{mbcescape} ) {
+              $val =~ s/([\x00-\x1f\/\\",=+<>#;\x7f-\xff])/
+                sprintf("\\%02x",ord($1))/xeg;
+            } else {
+              $val =~ s/([\x00-\x1f\/\\",=+<>#;])/
+                sprintf("\\%02x",ord($1))/xeg;
+            }
+            #escape leading and trailing whitespace
+            $val =~ s/(^\s+|\s+$)/
+              "\\20" x length $1/xeg; 
+          }
+          
+          # case fold attribute type and create return value
+          if ( !$opt{casefold} || $opt{casefold} eq 'upper' ) {
+            (uc $_)."=$val";
+          } elsif ( $opt{casefold} eq 'lower' ) {
+            (lc $_)."=$val";
+          } else {
+            "$_=$val";
+          }
+        } @types);
+    } @$rdns;
+  
+  # join RDNs into string, optionally reversing order
+  $opt{reverse}
+    ? join($separator, reverse @flatrdns)
+    : join($separator, @flatrdns);
+}
+
+
+=item ldap_explode_dn ( DN [ , OPTIONS ] )
+
+Explodes the given B<DN> into an array of hashes and returns a reference to this 
+array. Returns undef if B<DN> is not a valid Distinguished Name.
+
+A Distinguished Name is a sequence of Relative Distingushed Names (RDNs), which 
+themselves are sets of Attributes. For each RDN a hash is constructed with the 
+attribute type names as keys and the attribute values as corresponding values. 
+These hashes are then strored in an array in the order in which they appear 
+in the DN.
+
+For example, the DN 'OU=Sales+CN=J. Smith,DC=example,DC=net' is exploded to:
+[
+  {
+    'OU' =E<gt> 'Sales',
+    'CN' =E<gt> 'J. Smith'
+  },
+  {
+    'DC' =E<gt> 'example'
+  },
+  {
+    'DC' =E<gt> 'net'
+  }
+]
+
+(RFC2253 string) DNs might also contain values, which are the bytes of the 
+BER encoding of the X.500 AttributeValue rather than some LDAP string syntax. 
+These values are hex-encoded and prefixed with a #. To distingush such BER 
+values, ldap_explode_dn uses references to the actual values, 
+e.g. '1.3.6.1.4.1.1466.0=#04024869,DC=example,DC=com' is exploded to:
+[
+  {
+    '1.3.6.1.4.1.1466.0' =E<gt> \"\004\002Hi"
+  },
+  {
+    'DC' =E<gt> 'example'
+  },
+  {
+    'DC' =E<gt> 'com'
+  }
+];
+
+It also performs the following operations on the given DN:
+
+=over 4
+
+=item *
+
+Unescape "\" followed by ",", "+", """, "\", "E<lt>", "E<gt>", ";",
+"#", "=", " ", or a hexpair and and strings beginning with "#".
+
+=item *
+
+Removes the leading OID. characters if the type is an OID instead
+of a name.
+
+=back
+
+B<OPTIONS> is a list of name/value pairs, valid options are:
+
+=over 4
+
+=item casefold
+
+Controls case folding of attribute types names. Attribute values are not
+affected by this option. The default is to uppercase. Valid values are:
+	
+=over 4
+
+=item lower
+
+Lowercase attribute types names.
+
+=item upper
+
+Uppercase attribute type names. This is the default.
+
+=item none
+
+Do not change attribute type names.
+
+=item reverse
+
+If TRUE, the RDN sequence is reversed.
+
+=back
+
+=back
+
+=cut
+
+sub ldap_explode_dn($%) {
+  my ($dn, %opt) = @_;
+  return undef unless defined $dn;
+  return [] if $dn eq '';
+
+  my (@dn, %rdn);
   while (
-	 $dn =~ /\G(?:
-		\s*
-		([a-zA-Z][-a-zA-Z0-9]*|(?:[Oo][Ii][Dd]\.)?\d+(?:\.\d+)*)
-		\s*
-		=
-		\s*
-		(
-		  (?:[^\\",=+<>\#;]*[^\\",=+<>\#;\s]|\\(?:[\\ ",=+<>#;]|[0-9a-fA-F]{2}))*
-		  |
-		  \#(?:[0-9a-fA-F]{2})+
-		  |
-		  "(?:[^\\"]+|\\(?:[\\",=+<>#;]|[0-9a-fA-F]{2}))*"
-		)
-		\s*
-		(?:([;,+])\s*(?=\S)|$)
-		)\s*/gcx)
+  $dn =~ /\G(?:
+    \s*
+    ([a-zA-Z][-a-zA-Z0-9]*|(?:[Oo][Ii][Dd]\.)?\d+(?:\.\d+)*)
+    \s*
+    =
+    \s*
+    (
+      (?:[^\\",=+<>\#;]*[^\\",=+<>\#;\s]|\s*\\(?:[\\ ",=+<>#;]|[0-9a-fA-F]{2}))* 
+      |
+      \#(?:[0-9a-fA-F]{2})+
+      |
+      "(?:[^\\"]+|\\(?:[\\",=+<>#;]|[0-9a-fA-F]{2}))*"
+    )
+    \s*
+    (?:([;,+])\s*(?=\S)|$)
+    )\s*/gcx)
   {
     my($type,$val,$sep) = ($1,$2,$3);
 
-    $type =~ s/^oid\.(\d+(\.\d+)*)$/$1/i;
+    $type =~ s/^oid\.(\d+(\.\d+)*)$/$1/i; #remove leading "oid."
 
-    if ($val !~ /^#/) {
-      $val =~ s/^"(.*)"$/$1/;
-      $val =~ s/\\([\\ ",=+<>#;]|[0-9a-fA-F]{2})
-	       /length($1)==1 ? $1 : chr(hex($1))
-	       /xeg;
-      $val =~ s/([\\",=+<>#;])/\\$1/g;
-      $val =~ s/([\x00-\x1f\x7f-\xff])/sprintf("\\%02x",ord($1))/eg;
-
-      $val =~ s/(^\s+|\s+$)/"\\20" x length $1/ge;
+    if ( !$opt{casefold} || $opt{casefold} eq 'upper' ) {
+      $type = uc $type;
+    } elsif ( $opt{casefold} eq 'lower' ) {
+      $type = lc($type);
     }
 
-    push @rdn, "\U$type\E=$val";
+    if ( $val =~ s/^#// ) {
+      # decode hex-encoded BER value
+      my $tmp = pack('H*', $val);
+      $val = \$tmp;
+    } else {
+      # remove quotes
+      $val =~ s/^"(.*)"$/$1/;
+      # unescape characters
+      $val =~ s/\\([\\ ",=+<>#;]|[0-9a-fA-F]{2}) 
+           /length($1)==1 ? $1 : chr(hex($1))
+           /xeg; 
+    }
+
+    $rdn{$type} = $val;
 
     unless (defined $sep and $sep eq '+') {
-      push @dn, join("+", sort @rdn);
-      @rdn = ();
+      if ( $opt{reverse} ) {
+        unshift @dn, { %rdn };
+      } else {
+        push @dn, { %rdn };
+      }
+      %rdn = ();
     }
   }
 
-  (length($dn) != (pos($dn)||0))
-    ? undef
-    : join(",",@dn);
+  length($dn) == (pos($dn)||0)
+    ? \@dn
+    : undef;
 }
 
+
 =back
 
 =head1 AUTHOR
 
-Graham Barr <gbarr@pobox.com>
+Graham Barr E<lt>gbarr@pobox.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 1999-2000 Graham Barr. All rights reserved. This program is
+Copyright (c) 1999-2002 Graham Barr. All rights reserved. This program is
 free software; you can redistribute it and/or modify it under the same
 terms as Perl itself.
 
+ldap_explode_dn and canonical_dn also
+
+(c) 2002 Norbert Klasen, norbert.klasen@daasi.de, All Rights Reserved.
+
 =for html <hr>
 
-I<$Id: Util.pm,v 1.11 2001/04/12 16:40:40 gbarr Exp $>
+I<$Id: Util.pm,v 1.17 2003/05/08 09:27:41 gbarr Exp $>
 
 =cut
 
